# frozen_string_literal: true

module ErgomentumRspec
  module Matchers
    module ActiveModel
      def be_invalid_for(value)
        BeInvalidFor.new(value)
      end

      class BeInvalidFor
        def initialize(value)
          @value = value
        end

        def matches?(validator)
          model = Class.new do
            include ::ActiveModel::Model
            include ::ActiveModel::Attributes

            # ActiveModel sometimes needs a non nil class name.
            # Because we are using an anonymous class, which have no name by default, we have to override the name
            # method.
            def self.name
              "SomeClass"
            end

            attribute :some_attribute
            validates :some_attribute, validator
          end.new(some_attribute: @value)
          if model.valid?
            @message = "expecting '#{@value}' to be invalid"
            false
          elsif model.errors.messages[:some_attribute].include?(@expected_error_message)
            true
          else
            @message = "expecting error message '#{@expected_error_message}' for invalid value '#{@value}' but got "\
                       "'#{model.errors.messages[:some_attribute].join(', ')}'"
            false
          end
        end

        def and_add_error(expected_error_message)
          @expected_error_message = expected_error_message
          self
        end

        def failure_message
          @message
        end

        def description
          "'#{@value}' be a invalid value"
        end
      end
    end
  end
end
