# frozen_string_literal: true

module ErgomentumRspec
  module Matchers
    module ActiveModel
      def be_valid_for(value)
        BeValidFor.new(value)
      end

      class BeValidFor
        def initialize(value)
          @value = value
        end

        def matches?(validator)
          model = Class.new do
            include ::ActiveModel::Model
            include ::ActiveModel::Attributes

            # ActiveModel sometimes needs a non nil class name.
            # Because we are using an anonymous class, which have no name by default, we have to override the name
            # method.
            def self.name
              "SomeClass"
            end

            attribute :some_attribute
            validates :some_attribute, validator
          end.new(some_attribute: @value)
          model.valid? && model.errors.empty?
        end

        def failure_message
          "expect '#{@value}' to be valid"
        end

        def description
          "'#{@value}' be an valid value"
        end
      end
    end
  end
end
