# frozen_string_literal: true

module ErgomentumRspec
  module Matchers
    module Array
      def be_monotonically_decreasing
        MonotonicallyDecreasing.new
      end

      class MonotonicallyDecreasing
        def matches?(actual)
          @actual = actual
          derivative = actual.each_cons(2).map { |x, y| y <=> x }
          derivative.all? { |v| v <= 0 }
        end

        def failure_message
          "expected array #{@actual.inspect} to be monotonically decreasing"
        end

        def failure_message_when_negated
          "expected array #{@actual.inspect} to not be monotonically decreasing"
        end

        def description
          "be monotonically decreasing"
        end
      end
    end
  end
end
