# frozen_string_literal: true

module ErgomentumRspec
  module Matchers
    module Yield
      def yield_all(expected)
        YieldAll.new(expected)
      end

      class NoBlockGivenError < StandardError
        def initialize
          super("Block required. E.g. expect { |b| [0].each(&b) }.to yield_all(0)")
        end
      end

      class YieldAll
        include RSpec::Matchers::Composable

        def initialize(expected)
          @expected = expected
        end

        def supports_block_expectations?
          true
        end

        def matches?(*block)
          @count = 0
          @result = true
          if block_given?
            yield(method(:match))
          elsif block.first.is_a?(Proc)
            block.first.call(method(:match))
          else
            raise NoBlockGivenError
          end
          @result
        end

        def match(actual)
          @count += 1
          @last = actual
          @result &&= if @expected.respond_to?(:matches?)
                        @expected.matches?(actual)
                      else
                        values_match?(@expected, actual)
                      end
        end

        def failure_message
          if @expected.respond_to?(:failure_message)
            "Failed for #{ActiveSupport::Inflector.ordinalize(@count)} yield with:\n#{@expected.failure_message}"
          else
            "Expecting #{@expected} got #{@last} for #{ActiveSupport::Inflector.ordinalize(@count)} yield"
          end
        end

        def description
          "yields all"
        end
      end
    end
  end
end
